/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _CAM_DEBUG_H_
#define _CAM_DEBUG_H_

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>

G_BEGIN_DECLS
/*
#define LOG_TO_FILE
#define  ENABLE_TIME_MEASURE
#define TRACE_OBJECT
#define ENABLE_TIME_STAMP
*/
#define DEBUG_MESSAGE_ON
#define USE_DLOG_MESSAGE	/* for dlog */
#define  TRACE_FENTER_LEAVE
#define ENABLE_CHECK_MEMORY_OCCUPIED_SIZE
/* Domain */
#define LOG_UI	"UI"		/* about UI, EFL, */
#define LOG_MM	"MM"		/* about MSL, */
#define LOG_FILE "FILE"		/* about file */
#define LOG_SND "SOUND"
#define LOG_SYS "SYSTEM"
#define LOG_FWK "FRAMEWROK"
#define LOG_CONFIG "CONFIG"
#define LOG_CAM	"CAMERA"	/* #define LOG_CAMERA    0x00000001 */
#ifdef DEBUG_MESSAGE_ON
#define GET_FILE_NAME(p, f)	\
		do { \
			p = strrchr(f, '/'); 	\
			if (p)\
				p++;\
			else \
				p = f;\
		} while (0)
/*
#ifdef ENABLE_CHECK_MEMORY_STATUS
#define CHECK_MEMORY_STATS(s) fprintf(stderr, "[%s]: %s\n", s , __FUNCTION__); \
							      malloc_stats(); \
							      fprintf(stderr, "------------------\n")
#else
#define CHECK_MEMORY_STATS(s) 			G_STMT_START{ (void)0; }G_STMT_END
#endif
*/
#ifdef ENABLE_CHECK_MEMORY_OCCUPIED_SIZE
#include <malloc.h>
struct mallinfo g_mem_info;
#endif

#ifdef USE_DLOG_MESSAGE

#include <dlog.h>
#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "cam-app"

#define DEBUG_TRACE(fmt, arg...) do { if (TRUE) { \
			LOGD("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg);     \
		} } while (0)

#define debug_msg(domain, fmt, arg...)           LOGD("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)
#define cam_debug(domain, fmt, arg...)          LOGD("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)
#define cam_info(domain, fmt, arg...)            LOGI("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)
#define cam_message(domain, fmt, arg...)        LOGI("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)
#define cam_warning(domain, fmt, arg...)        LOGW("[%s : %s : %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)
#define cam_critical(domain, fmt, arg...)        LOGE("[%s : %s: %d] "fmt"\n", __FILE__, __FUNCTION__, __LINE__, ##arg)

#ifdef ENABLE_CHECK_MEMORY_OCCUPIED_SIZE
#define CHECK_MEMORY_STATS(s) g_mem_info = mallinfo(); \
								LOGD("\n\n[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.hblkhd + (g_mem_info).uordblks)
#endif
/*
								dlog(MID_CAMERA, DLOG_VERBOSE, "\n\n[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.arena); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.ordblks); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.hblks); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.hblkhd); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.uordblks); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.fordblks); \
								dlog(MID_CAMERA, DLOG_VERBOSE, "[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.keepcost); \
*/

#else				/*  #ifdef USE_DLOG_MESSAGE */

void
cam_log_func(const gchar *_domain, GLogLevelFlags log_level,
	     const char *file_name, const char *func, int line_number,
	     const char *format, ...);

#define debug_msg(category, fmt, arg...)		cam_log_func(category, G_LOG_LEVEL_DEBUG, __FILE__, __func__, __LINE__, fmt, ##arg)
#define cam_debug(category, fmt, arg...)		cam_log_func(category, G_LOG_LEVEL_DEBUG, __FILE__, __func__, __LINE__, fmt,  ##arg)
#define cam_info(category, fmt, arg...)		cam_log_func(category, G_LOG_LEVEL_INFO, __FILE__, __func__, __LINE__, fmt, ##arg)
#define cam_message(category, fmt, arg...)        cam_log_func(category, G_LOG_LEVEL_MESSAGE, __FILE__,  __func__, __LINE__, fmt, ##arg)
#define cam_warning(category, fmt, arg...) 	cam_log_func(category, G_LOG_LEVEL_WARNING, __FILE__,  __func__, __LINE__, fmt, ##arg)
#define cam_critical(category, fmt, arg...)	cam_log_func(category, G_LOG_LEVEL_CRITICAL, __FILE__, __func__, __LINE__, fmt, ##arg)

#ifdef ENABLE_CHECK_MEMORY_OCCUPIED_SIZE
#define CHECK_MEMORY_STATS(s) g_mem_info = mallinfo(); \
								cam_debug(LOG_SYS, "\n\n[[[[[[[[[[ %s : mem size is %d ]]]]]]]]]]\n", s, g_mem_info.hblkhd + (g_mem_info).uordblks)
#endif

#endif				/* DEBUG MESSAGE ON */

#else
#define debug_msg(domain, fmt, arg...)		G_STMT_START{ (void)0; } G_STMT_END
#define cam_debug(domain, fmt, arg...)		G_STMT_START{ (void)0; } G_STMT_END
#define cam_info(domain, fmt, arg...)		G_STMT_START{ (void)0; } G_STMT_END
#define cam_message(domain, fmt, arg...) G_STMT_START{ (void)0; } G_STMT_END
#define cam_warning(domain, fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define cam_critical(fmt, arg...)			G_STMT_START{ (void)0; } G_STMT_END
#ifdef ENABLE_CHECK_MEMORY_OCCUPIED_SIZE
#define CHECK_MEMORY_STATS(s) 			G_STMT_START{ (void)0; } G_STMT_END
#endif
#endif				/* #ifdef DEBUG_MESSAGE_ON */
#define warn_msg cam_warning
#define critical_msg cam_critical
/* Trace function enter/leave
 */
#ifdef TRACE_FENTER_LEAVE
#define debug_fenter(domain)				cam_debug(domain, " started");
#define debug_fleave(domain)				cam_debug(domain, " leaved");
#else
#define debug_fenter(domain)				G_STMT_START{ (void)0; } G_STMT_END
#define debug_fleave(domain)				G_STMT_START{ (void)0; } G_STMT_END
#endif
#define cam_ret_if(expr) do { \
	if (expr) { \
		return; \
	} \
} while (0)
#define cam_retv_if(expr, val) do { \
	if (expr) { \
		return (val); \
	} \
} while (0)
#define cam_retm_if(expr, fmt, arg...) do { \
	if (expr) { \
		cam_warning(LOG_UI, fmt, ##arg); \
		return; \
	} \
} while (0)
#define cam_retvm_if(expr, val, fmt, arg...) do { \
	if (expr) { \
		cam_warning(LOG_UI, fmt, ##arg); \
		return (val); \
	} \
} while (0)
/* Trace module elapsed time
 */
#ifdef ENABLE_TIME_MEASURE
    typedef enum {
	NORMAL_MEASURE_TYPE = 0,
	APP_LAUNCHING_TYPE,
	CAPTURE_MEASURE_TYPE,
	REGISTER_MEASURE_TYPE,
	REC_START_MEASURE_TYPE,
	REC_COMMIT_MEASURE_TYPE,
	REVIEW_MEASURE_TYPE,
	AUTO_FOCUSE_TIME,
	MEASURE_TYPE_MAX,
} cam_time_meature_type;

void debug_measure_start(cam_time_meature_type meature_type,
			 const gchar *format, ...);
void debug_measure_stop(cam_time_meature_type meature_type,
			const gchar *format, ...);
void debug_measure_init();
void debug_measure_close();

void debug_time_measure_start(const gchar *format, ...);
void debug_time_measure_stop(void);
void debug_time_measure_log(const gchar *format, ...);

#define debug_time_measure_fenter(fmt, arg...)	debug_time_measure_log("%s(). LINE :%d ", __FUNCTION__, __LINE__, ##arg)
#define debug_time_measure_fleave(fmt, arg...)	debug_time_measure_log("%s(). LINE :%d", __FUNCTION__, __LINE__, ##arg)

#else
#define debug_time_measure_start(fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_measure_stop()				G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_measure_log(fmt, arg...)		G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_measure_fenter(fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_measure_fleave(fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define debug_measure_start(fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define debug_measure_stop(fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#define debug_measure_init()		G_STMT_START{ (void)0; } G_STMT_END
#define debug_measure_close()	G_STMT_START{ (void)0; } G_STMT_END
#endif				/*ENABLE_TIME_MEASURE */
#ifdef ENABLE_TIME_STAMP
#include <sys/time.h>
#include <linux/unistd.h>
    enum {
	CAM_MSG_COLOR_DEFAULT = 0,
	CAM_MSG_COLOR_BLACK = 30,
	CAM_MSG_COLOR_RED = 31,
	CAM_MSG_COLOR_GREEN = 32,
	CAM_MSG_COLOR_YELLOW = 33,
	CAM_MSG_COLOR_BLUE = 34,
	CAM_MSG_COLOR_MAGENTA = 35,
	CAM_MSG_COLOR_CYAN = 36,
	CAM_MSG_COLOR_WHITE = 37,
};

static long s_time_sec = 0L;
static long s_time_usec = 0L;
static struct timeval s_cur_time;

static pid_t gettid(void)
{
	return syscall(__NR_gettid);
}

#define debug_time_reset(fmt, arg...)	\
	do {	\
		struct timeval time;	\
		gettimeofday(&time, NULL);	\
		s_cur_time = time; \
		s_time_sec = time.tv_sec;	\
		s_time_usec = time.tv_usec;	\
		debug_color(CAM_MSG_COLOR_RED, "[TIMER],<%03ld.%03ld>, \t000.000 sec (reset timer)\t000.000\t[%-20.20s:%4d]" fmt "\n", s_time_sec, s_time_usec, __FUNCTION__, __LINE__, ##arg);	\
	} while (0)

#define debug_time_stamp(fmt, arg...)	\
	do {	\
		struct timeval time;	\
		long sec, usec;	\
		long a_sec, a_usec; \
		gettimeofday(&time, NULL);	\
		a_sec = time.tv_sec - s_cur_time.tv_sec; \
		if (s_cur_time.tv_usec > time.tv_usec) {	\
			a_usec = 1000000L - s_cur_time.tv_usec + time.tv_usec;	\
			a_sec--;	\
		} else {	\
			a_usec = time.tv_usec - s_cur_time.tv_usec;	\
		}	\
		s_cur_time = time; \
		sec = time.tv_sec - s_time_sec;	\
		if (s_time_usec > time.tv_usec) {	\
			usec = 1000000L - s_time_usec + time.tv_usec;	\
			sec--;	\
		} else {	\
			usec = time.tv_usec - s_time_usec;	\
		}	\
		debug_color(CAM_MSG_COLOR_RED, "[TIMER],<%03ld.%03ld>, %03ld.%03ld sec (time stamp) %03ld.%03ld\t[%-20.20s:%4d]" fmt "\n", time.tv_sec, time.tv_usec, sec, usec / 1000, a_sec, a_usec/1000, __FUNCTION__, __LINE__, ##arg); \
	} while (0)

#define debug_time_stamp_simple(fmt, arg...)	\
	do {	\
		struct timeval time;	\
		gettimeofday(&time, NULL);	\
		debug_color(CAM_MSG_COLOR_RED, "[TIMER],<%03ld.%03ld> (reset timer), [%-20.20s:%4d], TID=%d" fmt "\n", time.tv_sec, time.tv_usec, __FUNCTION__, __LINE__, gettid(), ##arg);	\
	} while (0)

#define debug_color(color, fmt, arg...) \
	do {	\
		fprintf(stderr, "\033[%dm", color);	\
		fprintf(stderr, fmt, ##arg);	\
		fprintf(stderr, "\033[%dm", CAM_MSG_COLOR_DEFAULT);	\
		\
	} while (0)
#else
/*
 *	NO PROFILING MESSAGE
 */
#define debug_time_reset(fmt, arg...) G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_stamp(fmt, arg...) G_STMT_START{ (void)0; } G_STMT_END
#define debug_time_stamp_simple(fmt, arg...) G_STMT_START{ (void)0; } G_STMT_END
#endif
/* Trace GObject deletion
 */
#ifdef TRACE_OBJECT
void cam_debug_trace_object(gpointer obj, const gchar *format, ...);
#else
#define cam_debug_trace_object(obj, fmt, arg...)	G_STMT_START{ (void)0; } G_STMT_END
#endif
#define cam_debug_trace_alloc cam_debug_trace_object
    G_END_DECLS
#endif /*_CAM_DEBUG_H_*/
