/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include <glib.h>
#include <errno.h>

#ifndef CAMERROR_H_
#define CAMERROR_H_

#define CAM_ERROR_CLASS_COMMON							0x90000000
#define CAM_ERROR_CLASS_SYSTEM							0x90000100
#define CAM_ERROR_CLASS_FILE								0x90000200
#define CAM_ERROR_CLASS_MMCAMCORDER						0x90000400
#define CAM_ERROR_CLASS_MMPLAYER							0x90000800
#define CAM_ERROR_CLASS_MISC								0x90001600

enum {
	CAM_ERROR_NONE = 0,

	CAM_ERROR_BATTERY_NOT_ENOUGH = CAM_ERROR_CLASS_SYSTEM,
	CAM_ERROR_MULTIPLE_INSTANCE,
	CAM_ERROR_ENTER_REJECTED,
	CAM_ERROR_DEVICE,
	CAM_ERROR_STORAGE_UNAVAILABLE,
	CAM_ERROR_STORAGE_INSUFFICIENT,

	CAM_ERROR_FILE = CAM_ERROR_CLASS_FILE,
	CAM_ERROR_FILE_MANAGER,
	CAM_ERROR_FILE_MOVE_FAILED,
	CAM_ERROR_FILE_REGISTER_FAILED,
	CAM_ERROR_FILE_NOT_EXISTS,
	CAM_ERROR_FILE_RENAME_FAILED,
	CAM_ERROR_FILE_DELETE_FAILED,

	CAM_ERROR_MM_CAMCORDER = CAM_ERROR_CLASS_MMCAMCORDER,
	CAM_ERROR_AUTO_FOCUSING,
	CAM_ERROR_MISC = CAM_ERROR_CLASS_MISC,
	CAM_ERROR_QUERY_DB,
	CAM_ERROR_PANORAMA_LIB,
	CAM_ERROR_NCR_LIB,
	CAM_ERROR_NORMAL,
	CAM_ERROR_DEVICE_OFF,
	CAM_ERROR_MEMORY,
	CAM_ERROR_UNKNOWN,
};

#define CAMERA_ERROR_QUARK						g_quark_from_static_string("camera-error-quark")

#define cam_error_new(code, fmt, arg...)			g_error_new(CAMERA_ERROR_QUARK, code, fmt, ##arg)
#define cam_error_new_literal(code, msg)			g_error_new_literal(CAMERA_ERROR_QUARK, code, msg)
#define cam_set_error(error, code, fmt, arg...)	g_set_error(error, CAMERA_ERROR_QUARK, code, fmt, ##arg)

#endif				/* CAMERROR_H_ */
