/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include <app.h>
#include "cam_menu_composer.h"
#include "cam_debug.h"
#include "cam_capacity_type.h"
#include "cam_device_capacity.h"


const CAM_MENU_ITEM setting[] = {CAM_MENU_SHORTCUTS			, CAM_MENU_VOLUME_KEY		, CAM_MENU_BURST_SHOT_MODE		, CAM_MENU_SELF_PORTRAIT	, CAM_MENU_FLASH , CAM_MENU_SHOOTING_MODE
							, CAM_MENU_RECORDING_MODE		, CAM_MENU_SCENE_MODE		, CAM_MENU_EXPLOSURE_VALUE		, CAM_MENU_FOCUS_MODE		, CAM_MENU_TIMER
							, CAM_MENU_EFFECTS				, CAM_MENU_PHOTO_RESOLUTION	, CAM_MENU_VIDEO_RESOLUTION	 	, CAM_MENU_WHITE_BALANCE	, CAM_MENU_ISO
							, CAM_MENU_METERING				, CAM_MENU_ANTI_SHAKE		, CAM_MENU_OUTDOOR_VISIBILITY	, CAM_MENU_AUTO_CONTRAST	, CAM_MENU_GUIDELINES
							, CAM_MENU_CAPTURE_VOICE		, CAM_MENU_GPS_TAG			, CAM_MENU_SAVE_AS_FLIP			, CAM_MENU_STORAGE			, CAM_MENU_RESET};

const CAM_MENU_ITEM scene_mode[] = {CAM_MENU_SCENE_AUTO		, CAM_MENU_SCENE_PORTRAIT	, CAM_MENU_SCENE_LANDSCAPE	, CAM_MENU_SCENE_NIGHT	, CAM_MENU_SCENE_SPORTS
								, CAM_MENU_SCENE_PARTY		, CAM_MENU_SCENE_BEACHSNOW	, CAM_MENU_SCENE_SUNSET		, CAM_MENU_SCENE_DUSKDAWN, CAM_MENU_SCENE_FALL
								, CAM_MENU_SCENE_FIREWORK	, CAM_MENU_SCENE_TEXT		, CAM_MENU_SCENE_CANDLELIGHT, CAM_MENU_SCENE_BACKLIGHT, CAM_MENU_SCENE_SHOW_WINDOW};

const CAM_MENU_ITEM shooting_mode[] = {CAM_MENU_SHOOTING_SINGLE, CAM_MENU_SHOOTING_SELF_SINGLE};

const CAM_MENU_ITEM recording_mode[] = {CAM_MENU_RECORDING_MODE_NORMAL,			CAM_MENU_RECORDING_MODE_SLOW,	CAM_MENU_RECORDING_MODE_FAST,	CAM_MENU_RECORDING_MODE_LIMIT,
										CAM_MENU_RECORDING_MODE_SELF_NORMAL,	CAM_MENU_RECORDING_MODE_SELF_LIMIT};

const CAM_MENU_ITEM photo_resolution[] = {CAM_MENU_PHOTO_RESOLUTION_3264x2448		, CAM_MENU_PHOTO_RESOLUTION_3264x1960	, CAM_MENU_PHOTO_RESOLUTION_3264x1836	, CAM_MENU_PHOTO_RESOLUTION_2560x1920
								, CAM_MENU_PHOTO_RESOLUTION_2560x1536				, CAM_MENU_PHOTO_RESOLUTION_2560x1440	, CAM_MENU_PHOTO_RESOLUTION_2048x1536	, CAM_MENU_PHOTO_RESOLUTION_2048x1152
								, CAM_MENU_PHOTO_RESOLUTION_1920x1080				, CAM_MENU_PHOTO_RESOLUTION_1600x1200	, CAM_MENU_PHOTO_RESOLUTION_1392x1392	, CAM_MENU_PHOTO_RESOLUTION_1280x960
								, CAM_MENU_PHOTO_RESOLUTION_1280x720				, CAM_MENU_PHOTO_RESOLUTION_640x480};

const CAM_MENU_ITEM video_resolution[] = {CAM_MENU_VIDEO_RESOLUTION_FULLHD	, CAM_MENU_VIDEO_RESOLUTION_HD		, CAM_MENU_VIDEO_RESOLUTION_WVGA, CAM_MENU_VIDEO_RESOLUTION_VGA
								, CAM_MENU_VIDEO_RESOLUTION_QVGA			, CAM_MENU_VIDEO_RESOLUTION_QCIF};

const CAM_MENU_ITEM wb[] = {CAM_MENU_WB_AWB		,		CAM_MENU_WB_INCANDESCENT		,		CAM_MENU_WB_FLUORESCENT	,		CAM_MENU_WB_DAYLIGHT
						, CAM_MENU_WB_CLOUDY	, 		CAM_MENU_WB_SHADE			, 		CAM_MENU_WB_HORIZON		,		CAM_MENU_WB_FLASH
						, CAM_MENU_WB_CUSTOM};

const CAM_MENU_ITEM effect[] = {CAM_MENU_EFFECT_NONE	,	CAM_MENU_EFFECT_NEGATIVE	,	CAM_MENU_EFFECT_BLACKANDWHITE	,	CAM_MENU_EFFECT_SEPIA };

const CAM_MENU_ITEM focus_mode[] = {CAM_MENU_FOCUS_AUTO	,	CAM_MENU_FOCUS_MACRO	,	CAM_MENU_FOCUS_FACE_DETECTION	};

const CAM_MENU_ITEM flash[] = {CAM_MENU_FLASH_OFF	,	CAM_MENU_FLASH_ON	,	CAM_MENU_FLASH_AUTO	};

const CAM_MENU_ITEM metering[] = {CAM_MENU_METERING_MATRIX	,	CAM_MENU_METERING_CENTER_WEIGHTED	,	CAM_MENU_METERING_SPOT	};

const CAM_MENU_ITEM iso[] = {CAM_MENU_ISO_AUTO	,	CAM_MENU_ISO_50		,CAM_MENU_ISO_100, 	CAM_MENU_ISO_200
						,CAM_MENU_ISO_400	,	CAM_MENU_ISO_80O,	CAM_MENU_ISO_1600,	CAM_MENU_ISO_320O};

const CAM_MENU_ITEM storage[] = {CAM_MENU_STORAGE_PHONE, CAM_MENU_STORAGE_MMC};
const CAM_MENU_ITEM timer[] = {CAM_MENU_TIMER_OFF,CAM_MENU_TIMER_2SEC, CAM_MENU_TIMER_5SEC,CAM_MENU_TIMER_10SEC};
const CAM_MENU_ITEM volume_key[] = {CAM_MENU_VOLUME_KEY_ZOOM, CAM_MENU_VOLUME_KEY_SHUTTER};


static gboolean __is_enabled_burstshot(void *data);
static gboolean __is_enabled_flash(void* data);
static gboolean __is_enabled_shooting_mode(void* data);
static gboolean __is_enabled_recording_mode(void* data);
static gboolean __is_enabled_scene_mode(void* data);
static gboolean __is_enabled_explosure_value(void* data);
static gboolean __is_enabled_focus_mode(void* data);
static gboolean __is_enabled_timer(void* data);
static gboolean __is_enabled_effects(void* data);
static gboolean __is_enabled_photo_resolution(void* data);
static gboolean __is_enabled_video_resolution(void* data);
static gboolean __is_enabled_white_balance(void* data);
static gboolean __is_enabled_iso(void* data);
static gboolean __is_enabled_metering(void* data);
static gboolean __is_enabled_anti_shake(void* data);
static gboolean __is_enabled_rec_anti_shake(void* data);
static gboolean __is_enabled_outdoor_visibility(void* data);
static gboolean __is_enabled_auto_contrast(void* data);
static gboolean __is_enabled_capture_voice(void* data);
static gboolean __is_enabled_gps_tag(void* data);
static gboolean __is_enalbed_storage(void* data);
static gboolean __is_enabled_self_mode(void* data);
static gboolean __is_enabled_save_as_flip(void *data);


void cam_compose_setting_menu(void *data, cam_menu_composer* composer)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	cam_retm_if(composer == NULL, "composer is NULL");

	composer->title =  strdup(dgettext(PACKAGE, "IDS_CAM_BODY_SETTINGS"));
	composer->array = eina_array_new(25);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}

	eina_array_step_set(composer->array, sizeof(*(composer->array)), 5);

	unsigned int capacity = 0;
	GetCamDevFuncCaps(&capacity, data);

	/* Edit shortcuts */
	eina_array_push(composer->array, (void*)&setting[0]);

	/* Use volume key as */
	eina_array_push(composer->array, (void*)&setting[1]);

	if(capacity & CAM_CP_FUNC_BURST_SHOT_MODE)
		eina_array_push(composer->array, (void*)&setting[2]);

	if(capacity & CAM_CP_FUNC_SELF_MODE)
		eina_array_push(composer->array, (void*)&setting[3]);

	if(capacity & CAM_CP_FUNC_FLASH_MODE)
		eina_array_push(composer->array, (void*)&setting[4]);

	if(capacity & CAM_CP_FUNC_SHOT_MODE)
		eina_array_push(composer->array, (void*)&setting[5]);

	if(capacity & CAM_CP_FUNC_REC_MODE)
		eina_array_push(composer->array, (void*)&setting[6]);

	if(capacity & CAM_CP_FUNC_SCENE_MODE)
		eina_array_push(composer->array, (void*)&setting[7]);

	if(capacity & CAM_CP_FUNC_EXPOSURE)
		eina_array_push(composer->array, (void*)&setting[8]);

	if(capacity & CAM_CP_FUNC_FOCUS_MODE)
		eina_array_push(composer->array, (void*)&setting[9]);

	/* Timer */
	eina_array_push(composer->array, (void*)&setting[10]);

	if(capacity & CAM_CP_FUNC_EFFECT_MODE)
		eina_array_push(composer->array, (void*)&setting[11]);

	if(capacity & CAM_CP_FUNC_CAM_RESOLUTION)
		eina_array_push(composer->array, (void*)&setting[12]);

	if(capacity & CAM_CP_FUNC_REC_RESOLUTION)
		eina_array_push(composer->array, (void*)&setting[13]);

	if(capacity & CAM_CP_FUNC_WHITE_BALANCE)
		eina_array_push(composer->array, (void*)&setting[14]);

	if(capacity & CAM_CP_FUNC_ISO)
		eina_array_push(composer->array, (void*)&setting[15]);

	if(capacity & CAM_CP_FUNC_METERING)
		eina_array_push(composer->array, (void*)&setting[16]);

	if(capacity & CAM_CP_FUNC_CAM_ANS)
		eina_array_push(composer->array, (void*)&setting[17]);

	if(capacity & CAM_CP_FUNC_OUTDOOR)
		eina_array_push(composer->array, (void*)&setting[18]);

	if(capacity & CAM_CP_FUNC_AUTO_CONTRAST)
		eina_array_push(composer->array, (void*)&setting[19]);

	/* Guide lines */
	eina_array_push(composer->array, (void*)&setting[20]);

	/* Capture with voice */
//	eina_array_push(composer->array, (void*)&setting[21]);

	if(capacity & CAM_CP_FUNC_GPS)
		eina_array_push(composer->array, (void*)&setting[22]);

//	eina_array_push(composer->array, (void*)&setting[23]);

	if(capacity & CAM_CP_FUNC_STORAGE)
		eina_array_push(composer->array, (void*)&setting[24]);

	/* Setting Reset */
	eina_array_push(composer->array, (void*)&setting[25]);

}

void cam_compose_list_menu(void *data, int menu_type, cam_menu_composer* composer)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	cam_retm_if(composer == NULL, "composer is NULL");

	switch((CAM_MENU_ITEM)menu_type)
	{
		case CAM_MENU_VOLUME_KEY:
			cam_compose_menu_volume_key(data, composer);
			break;
		case CAM_MENU_SHOOTING_MODE:
			cam_compose_menu_shooting_mode(data, composer);
			break;
		case CAM_MENU_FLASH:
			cam_compose_menu_flash(data, composer);
			break;
		case CAM_MENU_RECORDING_MODE:
			cam_compose_menu_recording_mode(data, composer);
			break;
		case CAM_MENU_PHOTO_RESOLUTION:
			cam_compose_menu_photo_resolution(data, composer);
			break;
		case CAM_MENU_VIDEO_RESOLUTION:
			cam_compose_menu_video_resolution(data, composer);
			break;
		case CAM_MENU_ISO:
			cam_compose_menu_iso(data, composer);
			break;
		case CAM_MENU_FOCUS_MODE:
			cam_compose_menu_focus_mode(data, composer);
			break;
		case CAM_MENU_TIMER:
			cam_compose_menu_timer_mode(data, composer);
			break;
		case CAM_MENU_SCENE_MODE:
			cam_compose_menu_scene_mode(data, composer);
			break;
		case CAM_MENU_WHITE_BALANCE:
			cam_compose_menu_wb(data, composer);
			break;
		case CAM_MENU_EFFECTS:
			cam_compose_menu_effect(data, composer);
			break;
		case CAM_MENU_METERING:
			cam_compose_menu_metering(data, composer);
			break;
		case CAM_MENU_STORAGE:
			cam_compose_menu_storage(data, composer);
			break;
		default:
			cam_debug(LOG_UI, "invalid menu type");
			break;
	}
}

void cam_compose_menu_shooting_mode(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	composer->array = eina_array_new(10);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 5);

	unsigned int capacity = 0;
	GetCamDevShotModeCaps(&capacity, data);

	if(camapp->device_type == CAM_DEVICE_MEGA){
		if(capacity & CAM_CP_SHOT_MODE_SINGLE)
			eina_array_push(composer->array, (void*)&shooting_mode[0]);

	} else {
		if(capacity & CAM_CP_SHOT_MODE_SINGLE)
			eina_array_push(composer->array, (void*)&shooting_mode[9]);
		}
}

void cam_compose_menu_scene_mode(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(15);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}

	eina_array_step_set(composer->array, sizeof(*(composer->array)), 20);

	unsigned int capacity = 0;
	GetCamDevSceneModeCaps(&capacity, data);

	if(capacity & CAM_CP_SCENE_MODE_AUTO)
		eina_array_push(composer->array, (void*)&scene_mode[0]);

	if(capacity & CAM_CP_SCENE_MODE_PORTRAIT)
		eina_array_push(composer->array, (void*)&scene_mode[1]);

	if(capacity & CAM_CP_SCENE_MODE_LANDSCAPE)
		eina_array_push(composer->array, (void*)&scene_mode[2]);

	if(capacity & CAM_CP_SCENE_MODE_NIGHTSHOT)
		eina_array_push(composer->array, (void*)&scene_mode[3]);

	if(capacity & CAM_CP_SCENE_MODE_SPORTS)
		eina_array_push(composer->array, (void*)&scene_mode[4]);

	if(capacity & CAM_CP_SCENE_MODE_PARTY_AND_INDOOR)
		eina_array_push(composer->array, (void*)&scene_mode[5]);

	if(capacity & CAM_CP_SCENE_MODE_BEACH_AND_SNOW)
		eina_array_push(composer->array, (void*)&scene_mode[6]);

	if(capacity & CAM_CP_SCENE_MODE_SUNSET)
		eina_array_push(composer->array, (void*)&scene_mode[7]);

	if(capacity & CAM_CP_SCENE_MODE_DUSK_AND_DAWN)
		eina_array_push(composer->array, (void*)&scene_mode[8]);

	if(capacity & CAM_CP_SCENE_MODE_FALLCOLOR)
		eina_array_push(composer->array, (void*)&scene_mode[9]);

	if(capacity & CAM_CP_SCENE_MODE_FIREWORK)
		eina_array_push(composer->array, (void*)&scene_mode[10]);

	if(capacity & CAM_CP_SCENE_MODE_TEXT)
		eina_array_push(composer->array, (void*)&scene_mode[11]);

	if(capacity & CAM_CP_SCENE_MODE_CANDLE_LIGHT)
		eina_array_push(composer->array, (void*)&scene_mode[12]);

	if(capacity & CAM_CP_SCENE_MODE_BACK_LIGHT)
		eina_array_push(composer->array, (void*)&scene_mode[13]);

	if(capacity & CAM_CP_SCENE_MODE_SHOW_WINDOW)
		eina_array_push(composer->array, (void*)&scene_mode[14]);
}

void cam_compose_menu_photo_resolution(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(10);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 15);

	unsigned int capacity = 0;
	GetCamDevCamResolutionCaps(&capacity, data);


	if(capacity & CAM_CP_SIZE_3264X2448)
		eina_array_push(composer->array, (void*)&photo_resolution[0]);

	if(capacity & CAM_CP_SIZE_3264X1960)
		eina_array_push(composer->array, (void*)&photo_resolution[1]);

	if(capacity & CAM_CP_SIZE_3264X1836)
		eina_array_push(composer->array, (void*)&photo_resolution[2]);

	if(capacity & CAM_CP_SIZE_2560X1920)
		eina_array_push(composer->array, (void*)&photo_resolution[3]);

	if(capacity & CAM_CP_SIZE_2560X1536)
		eina_array_push(composer->array, (void*)&photo_resolution[4]);

	if(capacity & CAM_CP_SIZE_2560X1440)
		eina_array_push(composer->array, (void*)&photo_resolution[5]);

	if(capacity & CAM_CP_SIZE_2048X1536)
		eina_array_push(composer->array, (void*)&photo_resolution[6]);

	if(capacity & CAM_CP_SIZE_2048X1152)
		eina_array_push(composer->array, (void*)&photo_resolution[7]);

	if(capacity & CAM_CP_SIZE_1920X1080)
		eina_array_push(composer->array, (void*)&photo_resolution[8]);

	if(capacity & CAM_CP_SIZE_1600X1200)
		eina_array_push(composer->array, (void*)&photo_resolution[9]);

	if(capacity & CAM_CP_SIZE_1392X1392)
		eina_array_push(composer->array, (void*)&photo_resolution[10]);

	if(capacity & CAM_CP_SIZE_1280X960)
		eina_array_push(composer->array, (void*)&photo_resolution[11]);

	if(capacity & CAM_CP_SIZE_1280X720)
		eina_array_push(composer->array, (void*)&photo_resolution[12]);

	if(capacity & CAM_CP_SIZE_640X480)
		eina_array_push(composer->array, (void*)&photo_resolution[13]);

	int size = eina_array_count(composer->array);
	cam_critical(LOG_MM, "size %d ",size );
}

void cam_compose_menu_video_resolution(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(10);
	if (!(composer->array))	{
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 5);

	unsigned int capacity = 0;
	GetCamDevRecResolutionCaps(&capacity, data);

	if(capacity & CAM_CP_SIZE_1920X1080)
		eina_array_push(composer->array, (void*)&video_resolution[0]);

	if(capacity & CAM_CP_SIZE_1280X720)
		eina_array_push(composer->array, (void*)&video_resolution[1]);

	if(capacity & CAM_CP_SIZE_720X480)
		eina_array_push(composer->array, (void*)&video_resolution[2]);

	if(capacity & CAM_CP_SIZE_640X480)
		eina_array_push(composer->array, (void*)&video_resolution[3]);

	if(capacity & CAM_CP_SIZE_320X240)
		eina_array_push(composer->array, (void*)&video_resolution[4]);

	int size = eina_array_count(composer->array);
	cam_critical(LOG_MM, "size %d ",size );
}

void cam_compose_menu_effect(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevEffectCaps(&capacity, data);

	if(capacity & CAM_CP_EFFECT_NONE)
		eina_array_push(composer->array, (void*)&effect[0]);

	if(capacity & CAM_CP_EFFECT_NEGATIVE)
		eina_array_push(composer->array, (void*)&effect[1]);

	if(capacity & CAM_CP_EFFECT_MONO)
		eina_array_push(composer->array, (void*)&effect[2]);

	if(capacity & CAM_CP_EFFECT_SEPIA)
		eina_array_push(composer->array, (void*)&effect[3]);
}

void cam_compose_menu_wb(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevWBCaps(&capacity, data);

	if(capacity & CAM_CP_WB_AUTO)
		eina_array_push(composer->array, (void*)&wb[0]);

	if(capacity & CAM_CP_WB_INCANDESCENT)
		eina_array_push(composer->array, (void*)&wb[1]);

	if(capacity & CAM_CP_WB_FLUORESCENT)
		eina_array_push(composer->array, (void*)&wb[2]);

	if(capacity & CAM_CP_WB_DAYLIGHT)
		eina_array_push(composer->array, (void*)&wb[3]);

	if(capacity & CAM_CP_WB_CLOUDY)
		eina_array_push(composer->array, (void*)&wb[4]);

	if(capacity & CAM_CP_WB_SHADE)
		eina_array_push(composer->array, (void*)&wb[5]);

	if(capacity & CAM_CP_WB_HORIZON)
		eina_array_push(composer->array, (void*)&wb[6]);

	if(capacity & CAM_CP_WB_FLASH)
		eina_array_push(composer->array, (void*)&wb[7]);

	if(capacity & CAM_CP_WB_CUSTOM)
		eina_array_push(composer->array, (void*)&wb[8]);
}

void cam_compose_menu_focus_mode(void *data, cam_menu_composer* composer)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevFocusCaps(&capacity, data);

	if(capacity & CAM_CP_FOCUS_AUTO_FOCUS)
		eina_array_push(composer->array, (void*)&focus_mode[0]);

	if(capacity & CAM_CP_FOCUS_MACRO)
		eina_array_push(composer->array, (void*)&focus_mode[1]);

	if(!(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& camapp->camera_mode == CAM_CAMCORDER_MODE)){
		if(capacity & CAM_CP_FOCUS_FACE_DETECTION)
			eina_array_push(composer->array, (void*)&focus_mode[2]);
	}
}

void cam_compose_menu_iso(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(10);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 15);

	unsigned int capacity = 0;
	GetCamDevISOCaps(&capacity, data);

	if(capacity & CAM_CP_ISO_AUTO)
		eina_array_push(composer->array, (void*)&iso[0]);

	if(capacity & CAM_CP_ISO_50)
		eina_array_push(composer->array, (void*)&iso[1]);

	if(capacity & CAM_CP_ISO_100)
		eina_array_push(composer->array, (void*)&iso[2]);

	if(capacity & CAM_CP_ISO_200)
		eina_array_push(composer->array, (void*)&iso[3]);

	if(capacity & CAM_CP_ISO_400)
		eina_array_push(composer->array, (void*)&iso[4]);

	if(capacity & CAM_CP_ISO_800)
		eina_array_push(composer->array, (void*)&iso[5]);

#if 0
	if(capacity & CAM_CP_ISO_1600)
		eina_array_push(composer->array, (void*)&iso[6]);

	if(capacity & CAM_CP_ISO_3200)
		eina_array_push(composer->array, (void*)&iso[7]);
#endif
}

void cam_compose_menu_metering(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevMeteringCaps(&capacity, data);

	if(capacity & CAM_CP_METERING_MATRIX)
		eina_array_push(composer->array, (void*)&metering[0]);

	if(capacity & CAM_CP_METERING_CENTERWEIGHT)
		eina_array_push(composer->array, (void*)&metering[1]);

	if(capacity & CAM_CP_METERING_SPOT)
		eina_array_push(composer->array, (void*)&metering[2]);

}

void cam_compose_menu_recording_mode(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array))	{
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevRecModeCaps(&capacity, NULL);

	if(camapp->device_type == CAM_DEVICE_MEGA){
		if(capacity & CAM_CP_REC_MODE_NORMAL)
			eina_array_push(composer->array, (void*)&recording_mode[0]);

		if(capacity & CAM_CP_REC_MODE_SLOW_MOTION)
			eina_array_push(composer->array, (void*)&recording_mode[1]);

		if(capacity & CAM_CP_REC_MODE_FAST_MOTION)
			eina_array_push(composer->array, (void*)&recording_mode[2]);

		if(capacity & CAM_CP_REC_MODE_MMS)
			eina_array_push(composer->array, (void*)&recording_mode[3]);
	} else {
		if(capacity & CAM_CP_REC_MODE_NORMAL)
			eina_array_push(composer->array, (void*)&recording_mode[4]);

		if(capacity & CAM_CP_REC_MODE_MMS)
			eina_array_push(composer->array, (void*)&recording_mode[5]);
	}
}


void cam_compose_menu_flash(void *data, cam_menu_composer* composer)
{
	struct appdata *ad = (struct appdata *)data;
	cam_ret_if(ad == NULL);

	CamAppData *camapp = NULL;
	camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(5);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 10);

	unsigned int capacity = 0;
	GetCamDevFlashCaps(&capacity, data);

	if(capacity & CAM_CP_FLASH_OFF)
		eina_array_push(composer->array, (void*)&flash[0]);

	if(capacity & CAM_CP_FLASH_ON)
		eina_array_push(composer->array, (void*)&flash[1]);

	if(camapp->camera_mode == CAM_CAMERA_MODE){
		if(capacity & CAM_CP_FLASH_AUTO)
		eina_array_push(composer->array, (void*)&flash[2]);
	}

}

void cam_compose_menu_timer_mode(void *data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(4);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 8);

	eina_array_push(composer->array, (void*)&timer[0]);
	eina_array_push(composer->array, (void*)&timer[1]);
	eina_array_push(composer->array, (void*)&timer[2]);
	eina_array_push(composer->array, (void*)&timer[3]);
}

void cam_compose_menu_storage(void* data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(2);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 4);

	eina_array_push(composer->array, (void*)&storage[0]);
	eina_array_push(composer->array, (void*)&storage[1]);
}

void cam_compose_menu_volume_key(void* data, cam_menu_composer* composer)
{
	cam_retm_if(composer == NULL, "composer is NULL");

	composer->array = eina_array_new(2);
	if (!(composer->array)) {
		cam_critical(LOG_CAM, "could not create new eina array");
		return;
	}
	eina_array_step_set(composer->array, sizeof(*(composer->array)), 2);

	eina_array_push(composer->array, (void*)&volume_key[0]);
	eina_array_push(composer->array, (void*)&volume_key[1]);
}

void cam_compose_free(cam_menu_composer* composer)
{
	cam_ret_if(composer == NULL);

	if (composer->title)
		free(composer->title);

	if (composer->array)
		eina_array_free(composer->array);

	free(composer);
}

gboolean cam_is_enabled_menu(void *data, const CAM_MENU_ITEM item)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

	switch(item) {
		case CAM_MENU_SHORTCUTS:
			return TRUE;
		case CAM_MENU_BURST_SHOT_MODE:
			return __is_enabled_burstshot((void*)ad);
		case CAM_MENU_SELF_PORTRAIT:
			return __is_enabled_self_mode((void*)ad);
		case CAM_MENU_FLASH:
			return __is_enabled_flash((void*)ad);
		case CAM_MENU_SHOOTING_MODE:
			return __is_enabled_shooting_mode((void*)ad);
		case CAM_MENU_RECORDING_MODE:
			return __is_enabled_recording_mode((void*)ad);
		case CAM_MENU_SCENE_MODE:
			return __is_enabled_scene_mode((void*)ad);
		case CAM_MENU_EXPLOSURE_VALUE:
			return __is_enabled_explosure_value((void*)ad);
		case CAM_MENU_FOCUS_MODE:
			return __is_enabled_focus_mode((void*)ad);
		case CAM_MENU_TIMER:
			return __is_enabled_timer((void*)ad);
		case CAM_MENU_EFFECTS:
			return __is_enabled_effects((void*)ad);
		case CAM_MENU_PHOTO_RESOLUTION:
			return __is_enabled_photo_resolution((void*)ad);
		case CAM_MENU_VIDEO_RESOLUTION:
			return __is_enabled_video_resolution((void*)ad);
		case CAM_MENU_WHITE_BALANCE:
			return __is_enabled_white_balance((void*)ad);
		case CAM_MENU_ISO:
			return __is_enabled_iso((void*)ad);
		case CAM_MENU_METERING:
			return __is_enabled_metering((void*)ad);
		case CAM_MENU_ANTI_SHAKE:
			return __is_enabled_anti_shake((void*)ad);
		case CAM_MENU_OUTDOOR_VISIBILITY:
			return __is_enabled_outdoor_visibility((void*)ad);
		case CAM_MENU_AUTO_CONTRAST:
			return __is_enabled_auto_contrast((void*)ad);
		case CAM_MENU_GUIDELINES:
			return TRUE;
		case CAM_MENU_CAPTURE_VOICE:
			return __is_enabled_capture_voice((void*)ad);
		case CAM_MENU_SAVE_AS_FLIP:
			return __is_enabled_save_as_flip((void*)ad);
		case CAM_MENU_GPS_TAG:
			return __is_enabled_gps_tag((void*)ad);
		case CAM_MENU_STORAGE:
			return __is_enalbed_storage((void*)ad);;
		case CAM_MENU_RESET:
			return TRUE;
		case CAM_MENU_VOLUME_KEY:
			return TRUE;
		default:
			break;
		}

	return TRUE;

}

static gboolean __is_enabled_burstshot(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, NULL);

	if (!(capacity & CAM_CP_FUNC_BURST_SHOT_MODE))
		return FALSE;

	if ((camapp->shooting_mode != CAM_SINGLE_MODE)
			&& (camapp->shooting_mode != CAM_SELF_SINGLE_MODE))
		return FALSE;

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL)
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_flash(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, NULL);

	if(!(capacity & CAM_CP_FUNC_FLASH_MODE))
		return FALSE;

	if(ad->battery_status == LOW_BATTERY_WARNING_STATUS)
		return FALSE;

	if(!(camapp->scene_mode == CAM_SCENE_NONE
		|| camapp->scene_mode == CAM_SCENE_PORTRAIT
		|| camapp->scene_mode == CAM_SCENE_BACKLIGHT
		|| camapp->scene_mode == CAM_SCENE_PARTY
		|| camapp->scene_mode == CAM_SCENE_TEXT) )
		return FALSE;

	if (ad->torchlight_on)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_shooting_mode(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_SHOT_MODE))
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_recording_mode(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& camapp->camera_mode == CAM_CAMERA_MODE)
		return FALSE;

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& ad->exe_args->resolution == CAM_RESOLUTION_QCIF)
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_scene_mode(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_SCENE_MODE))
		return FALSE;

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& camapp->camera_mode == CAM_CAMCORDER_MODE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_explosure_value(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_EXPOSURE))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_focus_mode(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_FOCUS_MODE))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_timer(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	return TRUE;

}

static gboolean __is_enabled_effects(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_EFFECT_MODE))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_photo_resolution(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& camapp->camera_mode == CAM_CAMCORDER_MODE)
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_video_resolution(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(camapp->recording_mode == CAM_RECORD_MMS
		|| camapp->recording_mode == CAM_RECORD_SELF_MMS
		|| camapp->recording_mode == CAM_RECORD_SLOW
		|| camapp->recording_mode == CAM_RECORD_FAST)
		return FALSE;

	if(ad->launching_mode == CAM_LAUNCHING_MODE_EXTERNAL
		&& camapp->camera_mode == CAM_CAMERA_MODE)
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_white_balance(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_WHITE_BALANCE))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	return TRUE;
}

static gboolean __is_enabled_iso(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_ISO))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	if(camapp->camera_mode == CAM_CAMCORDER_MODE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_metering(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_METERING))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	if(camapp->camera_mode == CAM_CAMCORDER_MODE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_anti_shake(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if (!(capacity & CAM_CP_FUNC_CAM_ANS))
		return FALSE;

	if (camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_rec_anti_shake(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_REC_ANS))
		return FALSE;

	return TRUE;
}


static gboolean __is_enabled_outdoor_visibility(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_OUTDOOR))
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_auto_contrast(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_AUTO_CONTRAST))
		return FALSE;

	if(camapp->scene_mode != CAM_SCENE_NONE)
		return FALSE;


	return TRUE;

}

static gboolean __is_enabled_self_mode(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_SELF_MODE))
		return FALSE;
	return TRUE;

}


static gboolean __is_enabled_capture_voice(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_CAPTURE_VOICE))
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_gps_tag(void* data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_GPS))
		return FALSE;

	return TRUE;

}

static gboolean __is_enabled_save_as_flip(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retv_if(ad == NULL, FALSE);

	unsigned int capacity;
	GetCamDevFuncCaps(&capacity, (void*)ad);

	if(!(capacity & CAM_CP_FUNC_SAVE_AS_FLIP))
		return FALSE;

	return TRUE;

}


static gboolean __is_enalbed_storage(void* data)
{
	int noti_message;
	int ret = -1;

	ret = vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &noti_message);

	if ( ret != 0 ) // falied
	{
		cam_debug(LOG_CAM, "Get %s is failed", VCONFKEY_SYSMAN_MMC_STATUS);
		return FALSE;
	}

	if(noti_message==VCONFKEY_SYSMAN_MMC_MOUNTED)
	{
		return TRUE;
	}
	return FALSE;
}

