/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_ui_rotate_utils.h"
#include "cam_toolbar_edc_callback.h"
#include "cam_mm.h"
static void __cam_ui_rotate_utils_rotation_del_cb(void *data, Elm_Transit *transit)
{
	DEBUG_TRACE("rotate transition will be deleted");
}

static Ecore_Idler *__cam_ui_rotate_utils_idler = NULL;
static Eina_Bool __cam_ui_rotate_utils_obj_rotation_idler_cb(void *data)
{
	if (__cam_ui_rotate_utils_idler) {
		ecore_idler_del(__cam_ui_rotate_utils_idler);
		__cam_ui_rotate_utils_idler = NULL;
	}

	struct cam_ui_rotate_obj_params *rotate_obj_params = (struct cam_ui_rotate_obj_params *)data;
	if (!rotate_obj_params)
		return ECORE_CALLBACK_CANCEL;
	if (!rotate_obj_params->obj)
		return ECORE_CALLBACK_CANCEL;

	struct appdata *ad = (struct appdata *)cam_appdata_get();
	cam_retvm_if(ad == NULL, ECORE_CALLBACK_CANCEL, "appdata is NULL");

	Evas_Object *obj = rotate_obj_params->obj;

	////TODO:add a function, to unset popup
	Evas_Coord x, y, w, h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);
	static int repeat_count = 0;
	if (w == 0 || h == 0) {
		if (repeat_count++ == 10) {
			repeat_count = 0;
			return ECORE_CALLBACK_CANCEL;
		}
		DEBUG_TRACE("repeat_count = %d", repeat_count);
		return ECORE_CALLBACK_RENEW;
	}

	elm_object_part_content_unset(ad->toolbar_edje, "camera/setting/popup/area/toolbar");
	DEBUG_TRACE("rotate_obj_params->x = %d", rotate_obj_params->x);
	DEBUG_TRACE("rotate_obj_params->y = %d", rotate_obj_params->y);
	cam_ui_rotate_utils_map_rotate(obj, rotate_obj_params->x, rotate_obj_params->y, rotate_obj_params->degree);
	free(rotate_obj_params);
	rotate_obj_params = NULL;

	edje_object_signal_emit(_EDJ(obj), "set.popup.show", "prog");
	evas_object_raise(obj);
	evas_object_show(obj);

	return ECORE_CALLBACK_CANCEL;
}

void cam_ui_rotate_utils_map_setting_popup_rotate(Evas_Object *obj,
									Evas_Coord start_x,
									Evas_Coord start_y,
									double degree)
{
	DEBUG_TRACE("degree = %f", degree);
	struct cam_ui_rotate_obj_params *rotate_obj_params = calloc( 1, sizeof( struct cam_ui_rotate_obj_params ) );
	if (!rotate_obj_params) return;
	rotate_obj_params->obj = obj;
	rotate_obj_params->x = start_x;
	rotate_obj_params->y = start_y;
	rotate_obj_params->degree = degree;
	DEBUG_TRACE("rotate_obj_params->x = %d", start_x);
	DEBUG_TRACE("rotate_obj_params->y = %d", start_y);
	DEBUG_TRACE("rotate_obj_params->x = %d", start_x);
	DEBUG_TRACE("rotate_obj_params->degree = %f", degree);
	if (__cam_ui_rotate_utils_idler) {
		ecore_idler_del(__cam_ui_rotate_utils_idler);
		__cam_ui_rotate_utils_idler = NULL;

	}
	__cam_ui_rotate_utils_idler = ecore_idler_add(__cam_ui_rotate_utils_obj_rotation_idler_cb, (void *)rotate_obj_params);
}

void cam_ui_rotate_utils_map_rotate(Evas_Object *obj,
						Evas_Coord start_x,
						Evas_Coord start_y,
						double degree)
{
	if (!obj) return;
	Evas_Map *map = NULL;
	map = evas_map_new(4);
	if (!map) return;

	Evas_Coord x, y, w, h;
	float half_w, half_h;

	evas_map_util_points_populate_from_object_full(map, obj, 0);
	evas_object_geometry_get(obj, &x, &y, &w, &h);
	DEBUG_TRACE("obj x, y, w, h:%d %d %d %d", x, y, w, h);
	DEBUG_TRACE("start x, y, w, h: %d %d %d %d", start_x, start_y, w, h);
	half_w = (float)w * 0.5;
	half_h = (float)h * 0.5;
	evas_map_point_coord_set(map, 0, start_x, start_y, 0);
	evas_map_point_coord_set(map, 1, start_x + w , start_y, 0);
	evas_map_point_coord_set(map, 2, start_x + w , start_y + h, 0);
	evas_map_point_coord_set(map, 3, start_x, start_y + h, 0);
	evas_map_util_rotate(map, degree, start_x + half_w, start_y + half_h);
	/*note:foc 2000, just default*/
	evas_map_util_3d_perspective(map, start_x + half_w, start_y + half_h, 0, 2000);
	evas_object_map_enable_set(obj, EINA_TRUE);
	evas_object_map_set(obj, map);

	evas_map_free(map);
	map = NULL;

}

void cam_ui_rotate_utils_map_self_rotate(Evas_Object *obj,
							double degree)
{
	if (!obj) return;
	Evas_Map *map = NULL;
	map = evas_map_new(4);
	if (!map) return;

	Evas_Coord x, y, w, h;
	float half_w, half_h;

	evas_map_util_points_populate_from_object_full(map, obj, 0);
	evas_object_geometry_get(obj, &x, &y, &w, &h);
	DEBUG_TRACE("obj x, y, w, h:%d %d %d %d", x, y, w, h);
	half_w = (float)w * 0.5;
	half_h = (float)h * 0.5;
	evas_map_point_coord_set(map, 0, x, y, 0);
	evas_map_point_coord_set(map, 1, x + w , y, 0);
	evas_map_point_coord_set(map, 2, x + w , y + h, 0);
	evas_map_point_coord_set(map, 3, x, y + h, 0);
	evas_map_util_rotate(map, degree, x + half_w, y + half_h);
	/*note:foc 2000, just default*/
	evas_map_util_3d_perspective(map, x + half_w, y + half_h, 0, 2000);
	evas_object_map_enable_set(obj, EINA_TRUE);
	evas_object_map_set(obj, map);

	evas_map_free(map);
	map = NULL;

}


void cam_ui_rotate_utils_rotation_effect(Evas_Object *obj,
							float from_degree,
							float to_degree,
							double duration,
							void (*cb) (void *data, Elm_Transit *transit),
							void *data)
{
	DEBUG_TRACE();
	Evas_Object *nf = obj;
	Elm_Transit *transit = elm_transit_add();
	/*360 degree rotation effect with clock wise direction.*/
	elm_transit_object_add(transit, nf);
	elm_transit_effect_rotation_add(transit, from_degree, to_degree);
	elm_transit_duration_set(transit, duration);
	if (cb)
		elm_transit_del_cb_set(transit, cb, data);
	else
		elm_transit_del_cb_set(transit, __cam_ui_rotate_utils_rotation_del_cb, data);
	elm_transit_objects_final_state_keep_set(transit,  TRUE);
	elm_transit_go(transit);
}

/*note: below is for icon rotate*/

static gboolean cam_ui_rotate_utils_get_clockwise_state(struct appdata *ad)
{

	gboolean IsClockwise = FALSE;

	if (ad->rot_previous == APP_DEVICE_ORIENTATION_270) {
		if (ad->rot_current > ad->rot_previous){
			IsClockwise = TRUE;
		} else {
			IsClockwise = FALSE;
		}
	} else {
		if(ad->rot_current > ad->rot_previous) {
			IsClockwise = FALSE;
		} else {
			IsClockwise = TRUE;
		}
	}
	return IsClockwise;

}
void cam_ui_rotate_utils_get_rotation_degree(void* data, int* from_degree, int* to_degree)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	int tmp_to_degree = 0;
	int tmp_from_degree = 0;
	gboolean IsClockwise = FALSE;

	if (ad->rot_previous == APP_DEVICE_ORIENTATION_270) {
		if (ad->rot_current > ad->rot_previous){
			IsClockwise = TRUE;
		} else {
			IsClockwise = FALSE;
		}
	} else {
		if(ad->rot_current > ad->rot_previous) {
			IsClockwise = FALSE;
		} else {
			IsClockwise = TRUE;
		}
	}

	switch(ad->rot_previous) {
		case APP_DEVICE_ORIENTATION_270:
			if(ad->rot_current == APP_DEVICE_ORIENTATION_0) {
				tmp_from_degree = 360;
			} else {
				tmp_from_degree = 0;
			}
			break;
		case APP_DEVICE_ORIENTATION_0:
			tmp_from_degree = 270;
			break;
		case APP_DEVICE_ORIENTATION_90:
			tmp_from_degree = 180;
			break;
		case APP_DEVICE_ORIENTATION_180:
			tmp_from_degree = 90;
				break;
		default:
			break;
	}

	switch(ad->rot_current) {
		case APP_DEVICE_ORIENTATION_270:
			if( ad->rot_previous == APP_DEVICE_ORIENTATION_0) {
				tmp_to_degree = 360;
			} else {
				tmp_to_degree = 0;
			}
			break;
		case APP_DEVICE_ORIENTATION_0:
			tmp_to_degree = 270;
			break;
		case APP_DEVICE_ORIENTATION_90:
			tmp_to_degree = 180;
			break;
		case APP_DEVICE_ORIENTATION_180:
			tmp_to_degree = 90;
			break;
		default:
			break;
	}

	*from_degree = tmp_from_degree;
	*to_degree = tmp_to_degree;

	cam_debug(LOG_UI, "from_degree : %d, to_degree : %d ", *from_degree, *to_degree );
}


void cam_ui_rotate_utils_get_rotation_degree_map(void* data, int* degree)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	int tmp_degree = 0;

	gboolean IsClockwise = FALSE;

	if (ad->rot_previous == APP_DEVICE_ORIENTATION_270) {
		if (ad->rot_current > ad->rot_previous) {
			IsClockwise = TRUE;
		} else {
			IsClockwise = FALSE;
		}
	} else {
		if (ad->rot_current > ad->rot_previous) {
			IsClockwise = FALSE;
		} else {
			IsClockwise = TRUE;
		}
	}

	switch(ad->rot_current) {
	case APP_DEVICE_ORIENTATION_0:
		tmp_degree = 270;
		break;
	case APP_DEVICE_ORIENTATION_90:
		tmp_degree =  90;
		break;
	case APP_DEVICE_ORIENTATION_180:
		tmp_degree = 180;
		break;
	case APP_DEVICE_ORIENTATION_270:
		tmp_degree = 0;
		break;
	default:
		break;
	}

	*degree = tmp_degree;

	cam_debug(LOG_UI, "degree : %d", *degree );

}

static void __cam_ui_rotate_utils_focus_coordate_rotate(struct appdata *ad)
{
	/*TODO: for animator, we just rotate when popup create*/
	cam_retm_if(ad == NULL,  "appdata is NULL");

	int af_start_x, af_start_y;
	af_start_x = af_start_y = 0;

	int x_w, y_h;
	x_w = y_h = 0;
	if (ad->angle == 90
		|| ad->angle == 270) {
		x_w = 1280 * elm_config_scale_get();
		y_h = 720 * elm_config_scale_get();
	} else if (ad->angle == 0
		|| ad->angle == 180) {
		x_w = 720 * elm_config_scale_get();
		y_h = 1280 * elm_config_scale_get();
	} else {
		DEBUG_TRACE("%d", ad->angle_pre);
		x_w = 1280 * elm_config_scale_get();
		y_h = 720 * elm_config_scale_get();
	}

	switch (ad->rotate_angle) {
	case 0:
		af_start_x = ad->af_start_x;
		af_start_y = ad->af_start_y;
		break;
	case 90:/*un-clockwise*/
		af_start_x = ad->af_start_y;
		af_start_y = y_h - ad->af_start_x;
		if (ad->angle == 0 || ad->angle == 180) {
			af_start_y -= ad->camera_focus_w;
		} else {
			af_start_y -= ad->camera_focus_h;
		}
		break;
	case -90:
		af_start_x = x_w - ad->af_start_y;
		af_start_y = ad->af_start_x;
		if (ad->angle == 0 || ad->angle == 180) {
			af_start_x -= ad->camera_focus_h;
		} else {
			af_start_x -= ad->camera_focus_w;
		}
		break;
	case 180:
	case -180:
		af_start_x = x_w - ad->af_start_x;
		af_start_y = y_h - ad->af_start_y;
		if (ad->angle == 0 || ad->angle == 180) {
			af_start_x -= ad->camera_focus_h;
			af_start_y -= ad->camera_focus_w;
		} else {
			af_start_x -= ad->camera_focus_w;
			af_start_y -= ad->camera_focus_h;
		}
		break;
	default:
		DEBUG_TRACE("%d", ad->rotate_angle);
	}

	ad->af_start_x = af_start_x;
	ad->af_start_y = af_start_y;

}

static void __cam_ui_rotate_utils_after_rotate_cb(void *data, Elm_Transit *transit)
{

	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL,  "appdata is NULL");

	__cam_ui_rotate_utils_focus_coordate_rotate(ad);
	ad->angle_pre = ad->angle;/*must called after __cam_ui_roate_utils_focus_coordate_rotate*/
	cam_app_screen_rotate(ad);
	/*cam_screen_rotate(ad);*/
}

static void __cam_ui_rotate_utils_after_shortcuts_rotate_cb(void *data, Elm_Transit *transit)
{

	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL,  "appdata is NULL");
	cam_retm_if(ad->toolbar_edje == NULL,  "appdata is NULL");
	edje_object_signal_emit(_EDJ(ad->toolbar_edje),
					"toolbar_1_icon,normal", "prog");
	edje_object_signal_emit(_EDJ(ad->toolbar_edje),
					"toolbar_2_icon,normal", "prog");
	edje_object_signal_emit(_EDJ(ad->toolbar_edje),
					"toolbar_3_icon,normal", "prog");
	edje_object_signal_emit(_EDJ(ad->toolbar_edje),
					"toolbar_4_icon,normal", "prog");

}

gboolean cam_ui_rotate_utils_rotate_start(void *data, double duration)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE,  "appdata is NULL");

	if (ad->toolbar_setting_popup) {
		ad->show_setting_popup_after_rotate = TRUE;
		cam_toolbar_setting_popup_destroy(ad);
	}

	if (ad->setting_ctxpopup) {
		cam_toolbar_setting_popup_destroy(ad);
	}

	if (ad->edit_shortcuts_popup) {
		ad->show_edit_shortcuts_popup_after_rotate = TRUE;
		toolbar_destroy_shortcuts_popup(ad);
	}

	cam_debug(LOG_UI, "duration : %d", duration);

	ad->is_rotating = TRUE;
	Elm_Transit* transit= elm_transit_add();

	int from_degree = 0;
	int to_degree = 0;

	int i = 0;
	for (i = CAM_MENU_BAR_LEFT_0; i < CAM_MENU_BAR_LEFT_MAX_NUM; i++) {
		int type = ad->toolbar_menu_arrays[i].menu_type;
		Evas_Object *icon = ad->toolbar_menu_arrays[i].icon;
		if (icon == NULL) {
			cam_debug(LOG_UI, "toolbar_0_icon is null" );
		} else {
			elm_transit_object_add(transit, icon);
		}
	}

#if 0 /*TODO: block temporary, good solution should be consider*/
	if (ad->indicator_edje) {
		elm_transit_object_add(transit, ad->indicator_edje);
	}

#endif

	if (ad->focus_edje) {
		elm_transit_object_add(transit, ad->focus_edje);
	}
	if (ad->timer_icon_edje) {
		elm_transit_object_add(transit, ad->timer_icon_edje);
	}
	if (ad->capturebtn_edje) {
		elm_transit_object_add(transit, ad->capturebtn_edje);
	}
	if (ad->shutterbtn_camera_icon) {
		elm_transit_object_add(transit, ad->shutterbtn_camera_icon);
	}
	if (ad->recordingbtn) {
		elm_transit_object_add(transit, ad->recordingbtn);
	}
	if (ad->recording_stop_btn) {
		elm_transit_object_add(transit, ad->recording_stop_btn);
	}
	if (ad->toolbar.quickview_item.edje) {
		elm_transit_object_add(transit, ad->toolbar.quickview_item.edje);
	}
	if (ad->backbtn) {
		elm_transit_object_add(transit, ad->backbtn);
	}
	if (ad->caf_button) {
		elm_transit_object_add(transit, ad->caf_button);
	}

/*
	if (ad->recording_edje
		&& cam_mm_get_state() == RECORDER_STATE_RECORDING)
		elm_transit_object_add(transit, ad->recording_edje);*/
	cam_ui_rotate_utils_get_rotation_degree(ad, &from_degree, &to_degree);
	from_degree = ad->angle_pre;
	to_degree = ad->angle;

#if 0
	if ((from_degree == 0 && to_degree == 90)
	    || (from_degree == 90 && to_degree == 180)
	    || (from_degree == 180 && to_degree == 270)
	    || (from_degree == 270 && to_degree == 0)) {
		from_degree = 0;
		to_degree = -90;
		DEBUG_TRACE("CLOCKWISE");
	} else if ((from_degree == 90 && to_degree == 0)
		   || (from_degree == 180 && to_degree == 90)
		   || (from_degree == 270 && to_degree == 180)
		   || (from_degree == 0 && to_degree == 270)) {
		from_degree = 0;
		to_degree = 90;
		DEBUG_TRACE("UN-CLOCKWISE");
	}
#else
	DEBUG_TRACE("from_degree->to_degree %d->%d", from_degree, to_degree);

	if ((to_degree - from_degree) == 270) {
		from_degree = 0;
		to_degree = 90;
		ad->rotate_angle = 90;
	} else if ((to_degree - from_degree) == -270) {
		from_degree = 0;
		to_degree = -90;
		ad->rotate_angle = -90;
	} else {
		to_degree = (to_degree - from_degree) *(-1);
		from_degree = 0;
		ad->rotate_angle = to_degree;
	}
	DEBUG_TRACE("from_degree->to_degree %d->%d", from_degree, to_degree);
#endif
	cam_debug(LOG_UI, "ad->angle : %d, ad->angle_pre : %d ", ad->angle, ad->angle_pre );
	gboolean is_clockwise = cam_ui_rotate_utils_get_clockwise_state(ad);
	cam_debug(LOG_UI, "is_clockwise : %d, ", is_clockwise );
	elm_transit_effect_rotation_add(transit, from_degree, to_degree);
	elm_transit_duration_set(transit, duration);
	elm_transit_del_cb_set(transit, __cam_ui_rotate_utils_after_rotate_cb, data);
	elm_transit_objects_final_state_keep_set(transit, TRUE);
	elm_transit_go(transit);

	cam_debug(LOG_UI, "real_from_degree : %d, real_to_degree : %d ", from_degree, to_degree );
	/*ad->angle_pre = ad->angle;*/
	return FALSE;

}

gboolean cam_ui_rotate_utils_rotate_toolbar_shortcuts(void *data, double duration)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE,  "appdata is NULL");

	cam_debug(LOG_UI, "duration : %d", duration);

	Elm_Transit* transit= elm_transit_add();

	int from_degree = 0;
	int to_degree = 0;


	int i = 0;
	for (i = CAM_MENU_BAR_LEFT_0; i < CAM_MENU_BAR_LEFT_MAX_NUM; i++) {
		int type = ad->toolbar_menu_arrays[i].menu_type;
		Evas_Object *icon = ad->toolbar_menu_arrays[i].icon;
		if (icon == NULL) {
			cam_debug(LOG_UI, "toolbar_0_icon is null" );
		} else {
			elm_transit_object_add(transit, icon);
		}
	}

	cam_ui_rotate_utils_get_rotation_degree(ad, &from_degree, &to_degree);

	elm_transit_effect_rotation_add(transit, from_degree, to_degree);
	elm_transit_duration_set(transit, duration);
	elm_transit_del_cb_set(transit, __cam_ui_rotate_utils_after_shortcuts_rotate_cb, data);
	elm_transit_objects_final_state_keep_set(transit, TRUE);
	elm_transit_go(transit);

	cam_debug(LOG_UI, "real_from_degree : %d, real_to_degree : %d ", from_degree, to_degree );
	return FALSE;

}


//end file
